from fastapi import APIRouter, Query,  Body
import akshare as ak
import json
from pydantic import BaseModel
from src.models.user_strategy import *
from src.models.new_user_strategy import *
from src.akshare_data.service import *

router = APIRouter()  # 创建一个 FastAPI 路由器实例

# 数据库测试
@router.get("/userstrategy")
async def userstrategy():
    # 查询 user_id = 100096 的记录
    query = UserStrategy.filter(user_id=100096)  # 使用 Tortoise ORM 的查询方法
    data = await query.all()  # 异步获取所有结果
    return data

# 定义Pydantic模型
class InfoItem(BaseModel):
    code: str
    name: str
    market: str
    newprice: float
    amplitudetype: bool
    amplitude: float
    type: str

class MessageItem(BaseModel):
    id: str
    label: str
    value: str
    state: bool
    info: List[InfoItem]
    inputClass: str
    fixchange: bool

class StrategyRequest(BaseModel):
    id: int
    strategy_name: str
    message: List[MessageItem] or None
    info: List[InfoItem] or None

class MyData(BaseModel):
    mes: List[StrategyRequest]

# 新加的new_user_strategy数据库表
@router.post("/newuserstrategy")
async def newuserstrategy(strategy: StrategyRequest = Body(...)):
    # 获取请求数据
    request_data = strategy.dict()
    # 查询 new_user_strategy 表中是否存在相同的 id 和 strategy_name
    existing_strategy = await NewUserStrategy.filter(
        id=request_data['id'], strategy_name=request_data['strategy_name']
    ).first()

    if existing_strategy:
        # 如果已存在，直接返回提示信息
        return {"message": "该策略已存在"}

    # 查询 user_strategy 表中是否存在相同的 user_id 和 strategy_name
    existing_user_strategy = await UserStrategy.filter(
        user_id=request_data['id'], strategy_name=request_data['strategy_name']
    ).first()

    if existing_user_strategy:
        # 如果已存在，直接返回提示信息
        return {"message": "该策略已在 user_strategy 表中存在"}

    # 将 message 和 info 转换为 JSON 字符串
    message_json = json.dumps(request_data['message'])
    info_json = json.dumps(request_data['info'])

    # 创建新的 new_user_strategy 记录
    new_strategy = await NewUserStrategy.create(
        id=request_data['id'],
        strategy_name=request_data['strategy_name'],
        message=message_json,
        info=info_json
    )

    return {"message": "数据插入成功", "strategy_id": new_strategy.id}

@router.post("/newadd")
async def newadd(strategy: StrategyRequest = Body(...)):
    # 获取请求数据
    request_data = strategy.dict()
    # 查询 new_user_strategy 表中是否存在相同的 id 和 strategy_name
    existing_strategy = await NewUserStrategy.filter(
        id=request_data['id'], strategy_name=request_data['strategy_name']
    ).first()

    if existing_strategy:
        # 如果已存在，返回提示信息
        return {
            "code": 204,
            "message": "该分组已经存在"
        }

    # 将 message 和 info 转换为 JSON 字符串
    message_json = json.dumps(request_data['message'])
    info_json = json.dumps(request_data['info'])

    # 创建新的 new_user_strategy 记录
    await NewUserStrategy.create(
        id=request_data['id'],
        strategy_name=request_data['strategy_name'],
        message=message_json,
        info=info_json
    )
    return {
        "code": 200,
        "message": "新建分组成功！"
    }

# 获取数据
@router.get("/newget")
async def newget():
    # 查询 new_user_strategy 表中 id = 100096 的所有记录
    query = NewUserStrategy.filter(id=100096)
    data = await query.all()
    return {
        "code": 200,
        "data": data
    }

# 新增分组
@router.post("/newupdata")
async def newupdata(strategy: StrategyRequest = Body(...)):
    # 获取请求数据
    request_data = strategy.dict()

    # 将 message 和 info 转换为 JSON 字符串
    message_json = json.dumps(request_data['message'])
    info_json = json.dumps(request_data['info'])

    # 查询是否存在对应的 strategy_name
    existing_strategy = await NewUserStrategy.filter(strategy_name=request_data['strategy_name']).first()
    if not existing_strategy:
        return {"message": "不存在"}

    # 更新记录
    await NewUserStrategy.filter(strategy_name=request_data['strategy_name']).update(
        message=message_json,
        info=info_json
    )
    return {"message": "更新成功"}

class delItem(BaseModel):
    strategy_name: str
# 删除分组
@router.post("/newdel")
async def newdel(delitem: delItem):
    strategy_name = delitem.strategy_name

    # 删除 new_user_strategy 表中的记录
    delete_count_new = await NewUserStrategy.filter(strategy_name=strategy_name).delete()

    # 删除 user_strategy 表中的记录
    delete_count_user = await UserStrategy.filter(strategy_name=strategy_name).delete()

    # 检查是否删除成功
    if delete_count_new == 0 and delete_count_user == 0:
        return {"message": "未找到指定的策略名称"}
    return {"message": "删除成功"}


@router.get("/newmodify")
async def newmodify(
    strategy_name: str = Query(..., description="原始值"),
    new_strategy_name: str = Query(..., description="更改值")
):
    # 检查是否存在原始 strategy_name
    existing_strategy = await NewUserStrategy.filter(strategy_name=strategy_name).first()
    if not existing_strategy:
        return {"message": "原始策略名称不存在"}
    # 检查新策略名称是否已存在
    existing_new_strategy = await NewUserStrategy.filter(strategy_name=new_strategy_name).first()
    if existing_new_strategy:
        return {"message": "新策略名称已存在"}
    # 更新 strategy_name
    await NewUserStrategy.filter(strategy_name=strategy_name).update(strategy_name=new_strategy_name)

    return {"message": "重命名成功"}


# 股票数据
stock_data = None
@router.get("/stock")
async def stock(
        symbol: str = Query(..., description="股票代码"),
        start_date: str = Query(..., description="起始日期"),
        end_date: str = Query(..., description="结束日期"),
):
    # 获取股票日线行情数据
    global stock_data
    try:
        stock_zh_a_daily_df = ak.stock_zh_a_daily(symbol=symbol, start_date=start_date, end_date=end_date, adjust="qfq")

        dates_list = stock_zh_a_daily_df['date'].tolist()
        opens_list = clean_data(stock_zh_a_daily_df['open'].tolist())
        closes_list = clean_data(stock_zh_a_daily_df['close'].tolist())
        highs_list = clean_data(stock_zh_a_daily_df['high'].tolist())
        lows_list = clean_data(stock_zh_a_daily_df['low'].tolist())
        volumes_list = clean_data(stock_zh_a_daily_df['volume'].tolist())
        amounts_list = clean_data(stock_zh_a_daily_df['amount'].tolist())

        global stock_data
        stock_data = {
            "amount": amounts_list,
            "close": closes_list,
            "date": dates_list,
            "high": highs_list,
            "low": lows_list,
            "open": opens_list,
            "outstanding_share": [],
            "turnover": [],
            "volume": volumes_list
        }
    except Exception as e:
        print(e)
        print("无法使用该方式请求股票数据，当前股票可能不是A股")
        stock_data = {
            "amount": [],
            "close": [],
            "date": [],
            "high": [],
            "low": [],
            "open": [],
            "outstanding_share": [],
            "turnover": [],
            "volume": []
        }
    finally:
        return {"message": stock_data}

# 前端获取数据接口
@router.get("/kdata")
async def kdata():
    global stock_data
    if stock_data is None:
        stock_data = {
            "amount": [],
            "close": [],
            "date": [],
            "high": [],
            "low": [],
            "open": [],
            "outstanding_share": [],
            "turnover": [],
            "volume": []
        }
        return {"message": stock_data}
    else:
        return {"message": stock_data}

# ---------------------------------------------------------------------
# 港股代码数据
@router.get("/ganggudata")
async def ganggudata():
    stock_hk_spot_em_df = ak.stock_hk_spot_em()
    codes_list = stock_hk_spot_em_df['代码'].tolist()
    names_list = stock_hk_spot_em_df['名称'].tolist()
    price_list = clean_data(stock_hk_spot_em_df['最新价'].tolist())
    amplitudes_list = clean_data(stock_hk_spot_em_df['涨跌幅'].tolist())
    # 返回的数据
    ggstocking = []
    for i in range(9):
        if price_list[i] >= 0:
            flag = True
        else:
            flag = False
        ggstocking.append({
            'code': codes_list[i],
            'name': names_list[i],
            'market': '港股',
            'newprice': price_list[i],
            'amplitudetype': flag,
            'amplitude': amplitudes_list[i],
            'type': 'ganggu'
        })

    # 返回清理后的列表
    return ggstocking

# 港股K线图历史数据
@router.get("/ganggudataK")
async def ganggudataK(
        symbol: str = Query(..., description="股票代码"),
        start_date: str = Query(..., description="起始日期"),
        end_date: str = Query(..., description="结束日期"),
):
    try:
        stock_hk_hist_df = ak.stock_hk_hist(symbol=symbol, period="daily", start_date=start_date, end_date=end_date,
                                            adjust="")
        dates_list = stock_hk_hist_df['日期'].tolist()
        opens_list = clean_data(stock_hk_hist_df['开盘'].tolist())
        close_list = clean_data(stock_hk_hist_df['收盘'].tolist())
        high_list = clean_data(stock_hk_hist_df['最高'].tolist())
        low_list = clean_data(stock_hk_hist_df['最低'].tolist())
        volume_list = clean_data(stock_hk_hist_df['成交量'].tolist())
        amount_list = clean_data(stock_hk_hist_df['成交额'].tolist())
        global stock_data
        stock_data = {
            "amount": amount_list,
            "close": close_list,
            "date": dates_list,
            "high": high_list,
            "low": low_list,
            "open": opens_list,
            "outstanding_share": [],
            "turnover": [],
            "volume": volume_list
        }
    except Exception as e:
        print(e)
        stock_data = {
            "amount": [],
            "close": [],
            "date": [],
            "high": [],
            "low": [],
            "open": [],
            "outstanding_share": [],
            "turnover": [],
            "volume": []
        }
    finally:
        return {"message": stock_data}

# ---------------------------------------------------------------------
# 美股代码数据
@router.get("/meigudata")
async def meigudata():
    stock_us_spot_em_df = ak.stock_us_spot_em()
    codes_list = stock_us_spot_em_df['代码'].tolist()
    names_list = stock_us_spot_em_df['名称'].tolist()
    price_list = clean_data(stock_us_spot_em_df['最新价'].tolist())
    amplitudes_list = clean_data(stock_us_spot_em_df['涨跌幅'].tolist())

    # 返回的数据
    mgstocking = []
    for i in range(9):
        if price_list[i] >= 0:
            flag = True
        else:
            flag = False
        mgstocking.append({
            'code': codes_list[i],
            'name': names_list[i],
            'market': '美股',
            'newprice': price_list[i],
            'amplitudetype': flag,
            'amplitude': amplitudes_list[i],
            'type': 'meigu'
        })

    # 返回清理后的列表
    return mgstocking



# 美股K线图历史数据
@router.get("/meigudataK")
async def meigudataK(
        symbol: str = Query(..., description="股票代码"),
        start_date: str = Query(..., description="起始日期"),
        end_date: str = Query(..., description="结束日期"),
):
    try:
        stock_us_hist_df = ak.stock_us_hist(symbol=symbol, period="daily", start_date=start_date, end_date=end_date,
                                            adjust="qfq")
        dates_list = stock_us_hist_df['日期'].tolist()
        opens_list = stock_us_hist_df['开盘'].tolist()
        close_list = clean_data(stock_us_hist_df['收盘'].tolist())
        high_list = clean_data(stock_us_hist_df['最高'].tolist())
        low_list = clean_data(stock_us_hist_df['最低'].tolist())
        volume_list = clean_data(stock_us_hist_df['成交量'].tolist())
        amount_list = clean_data(stock_us_hist_df['成交额'].tolist())

        global stock_data
        stock_data = {
            "amount": amount_list,
            "close": close_list,
            "date": dates_list,
            "high": high_list,
            "low": low_list,
            "open": opens_list,
            "outstanding_share": [],
            "turnover": [],
            "volume": volume_list
        }
    except Exception as e:
        print(e)
        stock_data = {
            "amount": [],
            "close": [],
            "date": [],
            "high": [],
            "low": [],
            "open": [],
            "outstanding_share": [],
            "turnover": [],
            "volume": []
        }
    finally:
        return {"message": stock_data}

# ---------------------------------------------------------------------
# 沪深代码数据
@router.get("/hushendata")
async def hushendata():
    try:
        stock_zh_a_spot_df = ak.stock_kc_a_spot_em()
    except Exception as e:
        print(e)
    codes_list = stock_zh_a_spot_df['代码'].tolist()
    names_list = stock_zh_a_spot_df['名称'].tolist()
    price_list = clean_data(stock_zh_a_spot_df['最新价'].tolist())
    amplitudes_list = clean_data(stock_zh_a_spot_df['涨跌幅'].tolist())

    # 返回的数据
    hsstocking = []

    for i in range(9):
        if price_list[i] >= 0:
            flag = True
        else:
            flag = False
        hsstocking.append({
            'code': codes_list[i],
            'name': names_list[i],
            'market': '沪深',
            'newprice': price_list[i],
            'amplitudetype': flag,
            'amplitude': amplitudes_list[i],
            'type': 'hushen'
        })

    # 返回清理后的列表
    return hsstocking
@router.get("/hushendataK")
async def hushendataK(
        symbol: str = Query(..., description="股票代码"),
        start_date: str = Query(..., description="起始日期"),
        end_date: str = Query(..., description="结束日期"),
):
    try:
        stock_zh_a_daily_qfq_df = ak.stock_zh_a_daily(symbol='sh' + symbol, start_date=start_date, end_date=end_date,
                                                      adjust="qfq")

        # 如果你想要一个列表而不是Pandas Series
        dates_list = stock_zh_a_daily_qfq_df['date'].tolist()
        opens_list = clean_data(stock_zh_a_daily_qfq_df['open'].tolist())
        close_list = clean_data(stock_zh_a_daily_qfq_df['close'].tolist())
        high_list = clean_data(stock_zh_a_daily_qfq_df['high'].tolist())
        low_list = clean_data(stock_zh_a_daily_qfq_df['low'].tolist())
        volume_list = clean_data(stock_zh_a_daily_qfq_df['volume'].tolist())
        amount_list = clean_data(stock_zh_a_daily_qfq_df['amount'].tolist())

        global stock_data
        stock_data = {
            "amount": amount_list,
            "close": close_list,
            "date": dates_list,
            "high": high_list,
            "low": low_list,
            "open": opens_list,
            "outstanding_share": [],
            "turnover": [],
            "volume": volume_list
        }
    except Exception as e:
        print(e)
        stock_data = {
            "amount": [],
            "close": [],
            "date": [],
            "high": [],
            "low": [],
            "open": [],
            "outstanding_share": [],
            "turnover": [],
            "volume": []
        }
    finally:
        return {"message": stock_data}

